import numpy as np

class Label:
    def __init__ (self, items, name = "label"):
        self._origin = items
        self.name= name
        self._set = isinstance (self._origin, (list, tuple)) and list (set (self._origin)) or list (self._origin.keys ())
        self._set.sort ()
        self._indexes = {}
        for idx, item in enumerate (self._set):
            self._indexes [item] = idx
        self._items = dict ([(v, k) for k, v in self._indexes.items ()])        
    
    def __repr__ (self):
        return "<Label ({}): {}>".format (self.name, "[" + ", ".join ([str (each) for each in self._set]) + "]") 
    
    def __getitem__ (self, index):
        return self.item (index) 
        
    def info (self, item):
        return self._origin [item] 
    
    def __len__ (self):
        return len (self._set)
    
    def index (self, item):
        return self._indexes [item]
    
    def item (self, index):
        return self._items [index]
    
    def items (self):
        return self._set
    
    def top_k (self, arr, k = 1):
        items = []         
        for idx in np.argsort (arr)[::-1][:k]:
            items.append (self._items [idx])
        return items
    
    def setval (self, items, value = 1.0, type = np.float, prefix = None):
        arr = np.zeros (len (self._set)).astype (type)
        if not isinstance (items, (dict, list, tuple)):
            items = [items]
        for item in items:
            tid = self._indexes.get (item, -1)
            if tid == -1:
                continue    
            arr [self._indexes [item]] = 1
            
        if prefix is not None:
            return np.concatenate ([prefix, arr])
        else:
            return arr
        
    def onehot (self, items, type = np.float, prefix = None):
        return self.setval (items, 1.0, type, prefix)
    one = onehot
    
    def onehots (self, ys):
        return np.array ([self.one (y) for y in ys])
        
    
if __name__ == "__main__":    
    v = Vector (["a", "b", "c", "d", "e"])
    base = v.one (["c", "e"])
    
    v = Vector (["a", "b", "c", "d", "e"])
    print (v.one (["c", "e"], prefix = base))
    
    
    


# https://github.com/hwalsuklee/tensorflow-mnist-VAE

import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
from scipy.misc import imsave
from scipy.misc import imresize
from rs4 import pathtool
import shutil, os

class Plot_Reproduce_Performance():
    def __init__(self, DIR, n_img_x=8, n_img_y=8, img_w=28, img_h=28, resize_factor=1.0):
        self.DIR = DIR
        pathtool.mkdir (DIR)
        assert n_img_x > 0 and n_img_y > 0

        self.n_img_x = n_img_x
        self.n_img_y = n_img_y
        self.n_tot_imgs = n_img_x * n_img_y

        assert img_w > 0 and img_h > 0

        self.img_w = img_w
        self.img_h = img_h

        assert resize_factor > 0

        self.resize_factor = resize_factor
    
    def reset (self):
        if os.path.isdir (self.DIR):
            shutil.rmtree (self.DIR)
        pathtool.mkdir (self.DIR)
        
    def add_noise (self, images):        
        images = images * np.random.randint (2, size = images.shape)
        images += np.random.randint(2, size = images.shape)
        return images
        
    def save_images(self, images, name='result.jpg'):
        images = images.reshape(self.n_img_x*self.n_img_y, self.img_h, self.img_w)
        imsave(self.DIR + "/"+name, self._merge(images, [self.n_img_y, self.n_img_x]))

    def _merge(self, images, size):
        h, w = images.shape[1], images.shape[2]

        h_ = int(h * self.resize_factor)
        w_ = int(w * self.resize_factor)

        img = np.zeros((h_ * size[0], w_ * size[1]))

        for idx, image in enumerate(images):
            i = int(idx % size[1])
            j = int(idx / size[1])

            image_ = imresize(image, size=(w_,h_), interp='bicubic')

            img[j*h_:j*h_+h_, i*w_:i*w_+w_] = image_

        return img
    
    
class Plot_Manifold_Learning_Result(Plot_Reproduce_Performance):
    def __init__(self, DIR, n_img_x=20, n_img_y=20, img_w=28, img_h=28, resize_factor=1.0, z_range=2.0):
        self.DIR = DIR

        assert n_img_x > 0 and n_img_y > 0

        self.n_img_x = n_img_x
        self.n_img_y = n_img_y
        self.n_tot_imgs = n_img_x * n_img_y

        assert img_w > 0 and img_h > 0

        self.img_w = img_w
        self.img_h = img_h

        assert resize_factor > 0

        self.resize_factor = resize_factor

        assert z_range > 0
        self.z_range = z_range

        self._set_latent_vectors()

    def _set_latent_vectors(self):

        # z1 = np.linspace(-self.z_range, self.z_range, self.n_img_y)
        # z2 = np.linspace(-self.z_range, self.z_range, self.n_img_x)
        #
        # z = np.array(np.meshgrid(z1, z2))
        # z = z.reshape([-1, 2])

        # borrowed from https://github.com/fastforwardlabs/vae-tf/blob/master/plot.py
        z = np.rollaxis(np.mgrid[self.z_range:-self.z_range:self.n_img_y * 1j, self.z_range:-self.z_range:self.n_img_x * 1j], 0, 3)
        # z1 = np.rollaxis(np.mgrid[1:-1:self.n_img_y * 1j, 1:-1:self.n_img_x * 1j], 0, 3)
        # z = z1**2
        # z[z1<0] *= -1
        #
        # z = z*self.z_range

        self.z = z.reshape([-1, 2])

    # borrowed from https://github.com/ykwon0407/variational_autoencoder/blob/master/variational_bayes.ipynb
    def save_scattered_image(self, z, id, name='scattered_image.jpg'):
        N = 10
        plt.figure(figsize=(8, 6))
        plt.scatter(z[:, 0], z[:, 1], c=np.argmax(id, 1), marker='o', edgecolor='none', cmap=discrete_cmap(N, 'jet'))
        plt.colorbar(ticks=range(N))
        axes = plt.gca()
        axes.set_xlim([-self.z_range-2, self.z_range+2])
        axes.set_ylim([-self.z_range-2, self.z_range+2])
        plt.grid(True)
        plt.savefig(self.DIR + "/" + name)


# borrowed from https://gist.github.com/jakevdp/91077b0cae40f8f8244a
def discrete_cmap(N, base_cmap=None):
    """Create an N-bin discrete colormap from the specified input map"""

    # Note that if base_cmap is a string or None, you can simply do
    #    return plt.cm.get_cmap(base_cmap, N)
    # The following works for string, None, or a colormap instance:

    base = plt.cm.get_cmap(base_cmap)
    color_list = base(np.linspace(0, 1, N))
    cmap_name = base.name + str(N)
    return base.from_list(cmap_name, color_list, N)